#include "message.h"
#include "ui_message.h"
#include <QString>
#include <QtWebKit/QWebView>
#include <QByteArray>
#include <QtNetwork/QHttp>
#include <QtNetwork/QHttpRequestHeader>
#include <QFile>
#include <iostream>
#include <QtNetwork/QNetworkAccessManager>
#include <QtNetwork/QNetworkReply>
#include <QMessage>
#include <QtMessaging/QMessageManager>
#include <QtMessaging/QMessageFilter>
#include <QtMessaging/QMessageService>
#include <QtContacts/qcontactmanager.h>
#include <QtGlobal>
#include <QtDebug>
#include <QContactFilter>
#include <QContactDetailFilter>
#include <qtimer.h>
#include <QSettings>






QTM_USE_NAMESPACE

Message::Message(QWidget *parent) :
        QWidget(parent),
        ui(new Ui::Message)

{
     ui->setupUi(this);

    // Manager for listening messages
    m_manager = new QMessageManager(this);

    // Listen new added messages
    connect(m_manager, SIGNAL(messageAdded(const QMessageId&,
                              const QMessageManager::NotificationFilterIdSet&)),
            this, SLOT(messageAdded(const QMessageId&,
                       const QMessageManager::NotificationFilterIdSet&)));



    //connect new message signal
    //to parent's button pressed() signal to show message view.
    connect(m_manager, SIGNAL(messageAdded(const QMessageId&,
                              const QMessageManager::NotificationFilterIdSet&)),
                        parent, SLOT(changeViewToMessage()));
    connect(this, SIGNAL(new_translated_message()),
            parent, SLOT(changeViewToMessage()));



    connect(ui->ButtonGetContacts, SIGNAL(clicked()),
                        parent, SLOT(changeViewToContacts()));


    connect(ui->pushButtonSendMessage, SIGNAL(clicked()),
                        parent, SLOT(changeViewToSendMessage()));

     connect(this, SIGNAL(create_new_sms(QString, QString, QString)),
           parent, SLOT(load_new_sms(QString, QString, QString)));


    //set languages to default 'en'

    readSettings();
    //ui->langBox_1->setCurrentIndex(2);
    //lang_1 = "de";
    //lang_2 = "en";
    //orig_lang = "en";

    // Create 2 filers set for filtering messages
    // - SMS filter
    // - InboxFolder filter
    m_notifFilterSet.insert(m_manager->registerNotificationFilter(
        QMessageFilter::byType(QMessage::Sms) &
        QMessageFilter::byStandardFolder(QMessage::InboxFolder)));

    manager = new QNetworkAccessManager(this);





    // Listen for send message signal
    connect(ui->pushButtonSendMessage, SIGNAL(clicked()),
            this, SLOT(sendMessage()));

    connect(ui->pushButtonTranslate, SIGNAL(clicked()),
           this, SLOT(MakeLink()));





    //request if fully received
    connect(manager, SIGNAL(finished(QNetworkReply*)),
            this, SLOT(replyFinished(QNetworkReply*)));


    //read settings.

}



void Message::messageAdded(const QMessageId& id,
    const QMessageManager::NotificationFilterIdSet& matchingFilterIds)
{
    // Message added...
    if (matchingFilterIds.contains(m_notifFilterSet)) {
        // ...and it fits into our filters, lets process it
        m_messageId = id;
        QTimer::singleShot(0, this, SLOT(processIncomingSMS()));
    }
}



void Message::processIncomingSMS()
{

    QMessage message = m_manager->message(m_messageId);
    // SMS message body
    //QString messageString = message.textContent();
    origin_message = message.textContent();
    //origin_message = message.textContent();


    ui->Field_Message->setText(origin_message);
    //ui->Field_Message->setText("<font color=\"DimGray\">" + origin_message + "</font>");
    //ui->Field_Message->setText(messageString);
    QString SenderString  = message.from().addressee();

    //myLabel->setText( "<font color=\"red\">" + SenderString + "</font>");
    //ui->Field_From->setText(SenderString);
    //("<font color='red'>Some text</font>");
    //ui->Field_From->setText("<font color=\"silver\">" + SenderString + "</font>");
    ui->Field_From->setText(SenderString);
    //ui->Field_From->setText("<font color='red'>Some text</font>");

    //Create c_manager
    QContactManager *contactmanager = new QContactManager();


    QContactDetailFilter phoneFilter;
    phoneFilter.setDetailDefinitionName(QContactPhoneNumber::DefinitionName, QContactPhoneNumber::FieldNumber);
    phoneFilter.setValue(message.from().addressee());
    phoneFilter.setMatchFlags(QContactFilter::MatchExactly);
    QList<QContactLocalId> matchingContacts = contactmanager->contactIds(phoneFilter);
    if (matchingContacts.size() == 0) {
        qDebug() << "Incoming message from unknown contact (" << phoneFilter.value() << ")";
    } else {
        QContact match = contactmanager->contact(matchingContacts.at(0));
                    qDebug() << "Incoming message from"
                             << match.displayLabel()
                             << "(" << phoneFilter.value() << ")";

                    number_to = phoneFilter.value().toString();
                    ui->Field_From->setText(match.displayLabel());
                    //ui->Field_From->setText("<font color=\"Grey\">" + match.displayLabel() + "</font>");
                    //ui->Field_From->setText(match.displayLabel());
    }

    /* "Oma" kodi
        QList<QContact> matchingContacts = contactmanager->contacts(QContactPhoneNumber::match(message.from().addressee()));
        if(matchingContacts.size() != 0){
            ui->Field_From->setText(matchingContacts.at(0).displayLabel());
        }
           //QContactLocalId a = contactmanager->contact(matchingContacts.at(0));
    */
    MakeLink();


}

void Message::processIncomingSMS(QMessage message)
{
    // SMS message body
    QString messageString = message.textContent();
    ui->Field_Message->setText(messageString);
    QString SenderString  = message.from().addressee();
    ui->Field_From->setText(SenderString);
    getSender(&message);

}

QString Message::getSender(QMessage *message) {

    //Create c_manager
    QContactManager *contactmanager = new QContactManager();


    QContactDetailFilter phoneFilter;
    phoneFilter.setDetailDefinitionName(QContactPhoneNumber::DefinitionName, QContactPhoneNumber::FieldNumber);
    phoneFilter.setValue(message->from().addressee());
    phoneFilter.setMatchFlags(QContactFilter::MatchExactly);
    QList<QContactLocalId> matchingContacts = contactmanager->contactIds(phoneFilter);
    if (matchingContacts.size() == 0) {
        qDebug() << "Incoming message from unknown contact (" << phoneFilter.value() << ")";
        return "Incoming message from unknown contact";
    } else {
        QContact match = contactmanager->contact(matchingContacts.at(0));
                    qDebug() << "Incoming message from"
                             << match.displayLabel()
                             << "(" << phoneFilter.value() << ")";
                    ui->Field_From->setText(match.displayLabel());
        return (QString) match.displayLabel();
    }
}





Message::~Message()
{
    //save settings
    writeSettings();
    delete manager;
    delete ui;
}
void Message::replyFinished(QNetworkReply *my_reply)
{
    ui->Field_Translation->setText("");
    //ui->Field_Translation->setText("Reply Finnished");
    //read reply to buffer
    QString my_string = my_reply->readAll();
    QString size_of_buffer;
    size_of_buffer.setNum(my_string.size());
    //ui->Field_Translation->append("Chars received: " + size_of_buffer);

    //split source code
    if (my_string.contains("his.style.backgroundColor='#fff'\">"))
    {

        emit new_translated_message();


        if (my_string.contains("<h3 id=headingtext>"))
        {
            QStringList temp_lang_list = my_string.split("<h3 id=headingtext>");
            QString temp_lang = temp_lang_list[1];
            QStringList temp_lang_list_2 = temp_lang.split(" ");

            orig_lang = temp_lang_list_2[0];

            //ui->Field_Message->prev
            if (orig_lang == "English")
                orig_lang = "en";
            else if (orig_lang == "German")
                orig_lang = "de";
            else if (orig_lang == "Russian")
                orig_lang = "ru";
            else if (orig_lang == "Swedish")
                orig_lang = "sv";
            else if (orig_lang == "Finnish")
                orig_lang = "fi";
            else if (orig_lang == "French")
                orig_lang = "fr";
            ui->Field_Message->setText(orig_lang + ": " + origin_message);
            //ui->Field_Translation->append("Translated from " + orig_lang);

        }
        QStringList mylist = my_string.split("his.style.backgroundColor='#fff'\">", QString::SkipEmptyParts);
        //split source code
        QString temp_string = mylist[0];

        temp_string.setNum(temp_string.size());
        QString temp_new_string = mylist[1];



        QStringList my_list_2 = mylist[1].split("</span>", QString::SkipEmptyParts);
        //Return of module my_list_2[0]
//        ui->Field_Translation->append("------>  " + my_list_2[0]);

        //ui->Field_Translation->setText("<font color=\"white\">" + my_list_2[0] + "</font>");
        translated_message = ui->Field_Translation->text();
        ui->Field_Translation->setText(my_list_2[0]);
        //ui->textEdit->setText(my_list_2[0]);


    }
    else
        //ui->Field_Translation->setText("<font color=\"white\">" + origin_message + "</font>");
        ui->Field_Translation->setText(origin_message);





}

void Message::MakeLink()
{
    //'input' to store user input
    //QString input = ui->Field_Message->text();

    //language pair. 'auto' for automatic detection
    //QString lang_pair = ui->LangPair->text();
    if (origin_message.length() != 0)
    {

        //compose link
        //QString link = "http://translate.google.com/?hl=en&ie=UTF8&text=" + input + "&langpair=auto|"+ lang_1;
        QString link = "http://translate.google.com/?hl=en&ie=UTF8&text=" + origin_message + "&langpair=auto|"+ lang_1;

        //send request
        //system("ping google.fi");
        //system("espeak \"Test\"");
        //sudo apt-get -install espeak
        manager->get(QNetworkRequest(QUrl(link)));
    }

}


void Message::sendMessage()
{
    emit create_new_sms(number_to, lang_1, orig_lang);
    //QString address = "+47199";
    //QMessage message;
    //message.setType(QMessage::Sms);
    //message.setTo(QMessageAddress(QMessageAddress::System, address));
    //message.setBody("Hi! I'm a new message");
    //QMessageService* service = new QMessageService(this);
    //if (service->send(message)) {
      //  qDebug() << "Sended SMS to" << address;
        // Miksi alempi rivi ei näy?
        // ui->pushButtonSendMessage->setText("Send is completed");
    //}
}



void Message::on_langBox_1_currentIndexChanged(QString )
{
    lang_1 = ui->langBox_1->currentText();
    MakeLink();
    //translated_message();
}
//void Message::on_langBox_2_currentIndexChanged(QString )
//{
//    lang_2 = ui->langBox_2->currentText();
//}

void Message::writeSettings()
{
    QSettings settings("Grp1", "QuTe Translation");

    settings.beginGroup("Languages");
    //settings.setValue("lang_1", lang_1);
    settings.setValue("lang_1", ui->langBox_1->currentIndex());
    //settings.setValue("lang_2", lang_2);
    //settings.setValue("lang_2", ui->langBox_2->currentIndex());
    settings.endGroup();
}

void Message::readSettings()
{
    QSettings settings("Grp1", "QuTe Translation");

    settings.beginGroup("Languages");

    ui->langBox_1->setCurrentIndex(settings.value("lang_1").toInt());
    //ui->langBox_2->setCurrentIndex(settings.value("lang_2").toInt());
    settings.endGroup();
}
